from select import select
from socket import socketpair
from time import time
from struct import pack, unpack

from skypehub.models import Message

def message_logging_receiver(handler, message, status):
    Message.objects.create(
        body=message.Body,
        sender=message.Sender.Handle,
        chat_name=message.Chat.Name,
    )

class OnMessageHandler(object):
    """
    Skype OnMessage event handler
    """
    default_statuses = ('RECEIVED',)
    default_receivers = ()

    def __init__(self, skype=None):
        self.receivers = list(self.default_receivers)
        self.skype = skype

    def connect(self, receiver, statuses=None):
        if statuses is None:
            statuses = list(self.default_statuses)
        if not receiver in self.receivers:
            self.receivers.append((statuses, receiver))

    def dispatch(self, message, status):
        for statuses, receiver in self.receivers:
            if status in statuses:
                receiver(self, message, status)

class OnTimeHandler(object):
    """
    Skype OnTime event handler
    by moriyoshi
    """
    def __init__(self, skype=None):
        self.skype = skype
        self.pair = socketpair()
        self.timepoints = []
        self.callables = {}
        self._callables = {}
        self.id = 1

    def connect(self, callable, time):
        id = self.callables.get(callable, 0)
        if not id:
            id = self.id
            self.id += 1
            self.callables[callable] = id
            self._callables[id] = callable
        self.pair[1].send(pack("@ii", time, id))

    def __call__(self):
        wait = None
        while True:
            r, w, e = select([self.pair[0]], [], [], wait)
            if r:
                t, id = unpack("@ii", r[0].recv(8))
                i = self.search_nearest(t)
                self.timepoints.insert(i, (t, id))
                wait = max(self.timepoints[0][0] - time(), 0)
            else:
                _time, id = self.timepoints.pop(0)
                self._callables[id](self, _time)
                if self.timepoints:
                    wait = max(self.timepoints[0][0] - time(), 0)
                else:
                    wait = None

    def search_nearest(self, time):
        l = len(self.timepoints)
        if l == 0:
            return 0
        s = 0
        e = l

        while True:
            i = (s + e) // 2
            if self.timepoints[i][0] < time:
                s = i
                i = (i + e) // 2
                if i == l - 1:
                    i = l
                    break
            else:
                e = i
                i = (s + i) // 2
                if i == 0:
                    break
        return i


on_message = OnMessageHandler()
on_time = OnTimeHandler()
on_message.connect(message_logging_receiver)



from django.http import HttpResponse

try:
    import json
except ImportError:
    from django.utils import simplejson as json

from skypehub.decorators import skype_required
from skypehub.forms import PostMessageForm, PostUserMessageForm

def make_json_response(content):
    return HttpResponse(json.dumps(content, indent=2), content_type='application/javascript')

@skype_required
def list_chats(request, skype):
    chats = []
    for chat in skype.Chats:
        chats.append({
            'name': chat.Name,
            'topic': chat.Topic,
        })
    return make_json_response({'chats': chats})

@skype_required
def post_message(request, skype):
    form = PostMessageForm(request.POST or None)
    if form.is_valid():
        chat = skype.Chat(form.cleaned_data['chat'])
        chat.SendMessage(form.cleaned_data['message'])
        return make_json_response({'result': 'ok'})
    else:
        form = PostUserMessageForm(request.POST or None)
        if form.is_valid():
            chat = skype.CreateChatWith(form.cleaned_data['username'])
            chat.SendMessage(form.cleaned_data['message'])
            return make_json_response({'result': 'ok'})

    return  make_json_response({'result': 'validation error.'})



# coding: utf-8
from django.db import models

class Message(models.Model):
    body = models.TextField(u'本文')
    sender = models.CharField(u'送信者', max_length=200)
    chat_name = models.CharField(u'チャット名', max_length=100)
    ctime = models.DateTimeField(u'作成日時', auto_now_add=True)

    def __unicode__(self):
        return self.body

    class Meta:
        ordering = ('-ctime',)
        verbose_name = u'スカイプメッセージ'
        verbose_name_plural = u'スカイプメッセージ'



from django.conf.urls.defaults import *

urlpatterns = patterns('skypehub.views',
    url(r'^list_chats$', 'list_chats', name='skype_list_chats'),
    url(r'^post_message', 'post_message', name='skype_post_message'),
)



from django.contrib import admin

from skypehub.models import Message

class MessageModelAdmin(admin.ModelAdmin):
    list_display = ('sender', 'chat_name', 'body', 'ctime')
    list_filter = ('chat_name', 'sender', 'ctime')
    search_fields = ('sender', 'chat_name', 'body')

admin.site.register(Message, MessageModelAdmin)



import os
import time
import imp
from optparse import make_option

from django.conf import settings
from django.core.management import BaseCommand
from django.utils.daemonize import become_daemon
from django.utils.importlib import import_module

from skypehub.utils import get_skype, SKYPE_HOOK_OPTIONS

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--daemonize', action='store_true',
                dest='daemonize', help='daemonize bot.'),
        make_option('--pidfile', dest='pidfile', default=None,
                help='create pid file.'),
    )
    help = "run skype bot."

    def handle(self, *args, **options):
        skype_options = getattr(settings, 'SKYPE_HOOK_OPTIONS', SKYPE_HOOK_OPTIONS)
        skype = get_skype(**skype_options)

        # --daemonize
        if options['daemonize']:
            become_daemon()

        # --pidfile
        if options['pidfile']:
            pidfile = open(options['pidfile'], 'w')
            pidfile.write('%d' % os.getpid())
            pidfile.close()

        from skypehub.handlers import on_message, on_time

        # load module
        for app in settings.INSTALLED_APPS:
            try:
                app_path = import_module(app).__path__
            except AttributeError:
                continue
            try:
                imp.find_module('skypebot', app_path)
            except ImportError:
                continue
            import_module("%s.skypebot" % app)

        # attach skype
        on_message.skype = skype
        on_time.skype = skype
        skype.OnMessageStatus = on_message.dispatch

        skype.Attach()
        on_time()



from django.test import TestCase

from skypehub.decorators import skype_required

class SkypeTest(TestCase):
    def test_get_skype(self):
        decorated = skype_required(lambda req, skype: skype)
        self.assertNotEqual(decorated, None)



from django.conf import settings

from skypehub.utils import get_skype

def skype_required(view):
    def _inner(request, *args, **kwargs):
        hook_options = getattr(settings, 'SKYPE_HOOK_OPTIONS',  {})
        skype = get_skype(**hook_options)
        return view(request, skype, *args, **kwargs)
    return _inner



from django import forms

class PostMessageForm(forms.Form):
    message = forms.CharField()
    chat = forms.CharField()

class PostUserMessageForm(forms.Form):
    message = forms.CharField()
    username = forms.CharField()
